#include <algorithm>
#include "common.h"
#include <commctrl.h>
#include <shellapi.h>
#include "../current/lsapi/lsapi.h"
#include "../current/lsapi/macros.h"
#include "utilities.h"
#include "taskbar.h"
#include "bangs.h"
#include "taskbarbutton.h"

extern Taskbar *taskbar;

extern vector<string> NoFlashList;
extern vector<string> RemoveList;
extern vector<string> HideList;
extern vector<string> AddList;
extern vector<string> NoScrollList;

TaskbarButton::TaskbarButton( Taskbar &tb, HWND hWnd )
: container(tb), skin(NULL), activeskin(NULL), flashskin(NULL), minimizedskin(NULL),
  otherCaption(NULL), caption(NULL), /*mousePressed(FALSE),*/ mouseOver(FALSE),
  taskflash(FALSE), x(0), y(0), height(0), width(0), flashing(false), flashingFlashSkin(false),
  currentState(STATE_UNKNOWN), hTask(hWnd)
{
	//20021110
	/*hTask = hWnd;
	skin = NULL;
	activeskin = NULL;
	flashskin = NULL;
	minimizedskin = NULL;

	otherCaption = NULL;
	caption = NULL;*/

	LONG tmpLong;
	if(tmpLong = GetWindowLong(hWnd, GWL_EXSTYLE))
		isOnTop = ((tmpLong & WS_EX_TOPMOST) == WS_EX_TOPMOST );
	else
		isOnTop = false;

	if (hTask)
	{
		caption = GetWindowCaption();
		// add tooltip
		TOOLINFO ti;
		memset( &ti, 0, sizeof(TOOLINFO) );

		ti.cbSize = sizeof(TOOLINFO);
		ti.hwnd = container.hWnd;
		ti.uId = (UINT) this;
		ti.lpszText = caption;

		if(container.useToolTips)
		{
			SendMessage( container.hToolTips, TTM_ADDTOOL, 0, (LPARAM) &ti );
			SetWindowPos( container.hToolTips, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);
		}
	}
	if(taskbar->showIcon)
		icon = GetIcon();

	//mousePressed = FALSE;
	//20021110
	/*mouseOver = FALSE;
	taskflash = FALSE;

	x = 0;
	y = 0;
	height = 0;
	width = 0;

	flashing = false;*/
}

TaskbarButton::~TaskbarButton()
{
	delete caption;
	delete otherCaption;

	// delete tooltip
	TOOLINFO ti;
	memset( &ti, 0, sizeof(TOOLINFO) );

	ti.cbSize = sizeof(TOOLINFO);
	ti.hwnd = container.hWnd;
	ti.uId = (UINT) this;

	if(taskbar->useToolTips)//20021108
		SendMessage( container.hToolTips, TTM_DELTOOL, 0, (LPARAM) &ti );
}

BOOL TaskbarButton::HasPoint( int xP, int yP )
{
	if ( xP < (x + width) && xP >= x )
	{
		if ( yP < (y + height) && yP >= y )
			return TRUE;
	}

	return FALSE;
}

void TaskbarButton::Move( int xP, int yP, int widthP, int heightP )
{
	x = xP;
	y = yP;
	height = heightP;
	width = widthP;

	// update tooltip
	TOOLINFO ti;
	memset( &ti, 0, sizeof(TOOLINFO) );

	ti.cbSize = sizeof(TOOLINFO);
	ti.hwnd = container.hWnd;
	ti.uId = (UINT) this;
	ti.rect.left = x;
	ti.rect.top = y;
	ti.rect.right = x + width;
	ti.rect.bottom = y + height;

	if(container.useToolTips)
	{
		SendMessage( container.hToolTips, TTM_NEWTOOLRECT, 0, (LPARAM) &ti );
		SetWindowPos( container.hToolTips, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);
	}
}



void TaskbarButton::OnLButtonDown( int, int )
{
	switch (taskbar->TaskBarTileLButton){
		case MOUSE_SELECT:
			SelectDown();
		    break;
		case MOUSE_MENU:
			MenuDown();
		    break;
		case MOUSE_ONTOP:
			OnTopToggle();
			break;
		case MOUSE_CLOSE:
			Close();
		    break;
		case MOUSE_CLOSE2:
			Close2();
		    break;
		case MOUSE_NONE:
		    break;
        default:
		    break;
	}
}

void TaskbarButton::OnTopToggle()
{
	if (!isOnTop){
		EndDeferWindowPos(DeferWindowPos(BeginDeferWindowPos(1) ,this->hTask, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE|SWP_NOSIZE));//Set ontop
		isOnTop = TRUE;
		// undocumented call to switch window activation
		SwitchToThisWindow( hTask, 1 );
	}
	else{
		EndDeferWindowPos(DeferWindowPos(BeginDeferWindowPos(1) ,this->hTask, HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOMOVE|SWP_NOSIZE));//Set not ontop
		isOnTop = FALSE;
	}
}


void TaskbarButton::Close()
{
	//SendMessage(this->hTask, WM_CLOSE,0,0);
	//SendNotifyMessage(this->hTask, WM_CLOSE,0,0);
	PostMessage(this->hTask, WM_CLOSE,0,0);
}

void TaskbarButton::Close2()
{
	//SendMessage(this->hTask, WM_DESTROY,0,0); //if cancel -> still destroyed
	PostMessage(this->hTask, WM_DESTROY,0,0); //if cancel -> still destroyed
}

void TaskbarButton::SelectUp(){
	//mousePressed = FALSE;

	if ( container.GetCapture() != this ){
		return ;
	}

	container.ReleaseCapture();

	if ( !mouseOver )
	{
		container.Invalidate( x, y, width + 1, height + 1 );
		return ;
	}
}

void TaskbarButton::SelectDown(){
	container.SetCapture( this );

	//mousePressed = TRUE;

	if(this->currentState == STATE_FLASHING){
			this->SetFlashing(FALSE);
			this->currentState = this->getCurrentState(false);
	}
	switch(this->currentState){
		case STATE_MINIMIZED:
			SwitchToThisWindow( hTask, SW_RESTORE );
			break;
		case STATE_SELECTED:
			ShowWindowAsync( hTask, SW_MINIMIZE );
			break;
		default:
			SendMessage( GetLitestepWnd(), LM_BRINGTOFRONT, 0, (LPARAM) hTask );
			//20021110
		/*default:
			{
				SendMessage( GetLitestepWnd(), LM_BRINGTOFRONT, 0, (LPARAM) hTask );
				SwitchToThisWindow( hTask, SW_RESTORE );
			}*/
	}

	//20021112
	//	container.lastActive ????
	container.OnRedraw(container.hWnd, true);
	//container.Invalidate( x, y, width + 1, height + 1 );
}

void TaskbarButton::MenuUp(){
	//mousePressed = FALSE;
	if ( container.GetCapture() != this )
		return ;

	container.ReleaseCapture();

	if ( !mouseOver )
	{
		container.Invalidate( x, y, width + 1, height + 1 );
		return ;
	}

	HMENU systemMenu;
	systemMenu = GetSystemMenu( hTask, FALSE );

	WINDOWPLACEMENT wp;
	wp.length = sizeof(WINDOWPLACEMENT);
	GetWindowPlacement( hTask, &wp );

	// restore is enabled only if minimized or maximized (not normal)
	EnableMenuItem( systemMenu, SC_RESTORE, MF_BYCOMMAND |
	                (wp.showCmd != SW_SHOWNORMAL ? MF_ENABLED : MF_GRAYED) );

	// move is enabled only if normal
	EnableMenuItem( systemMenu, SC_MOVE, MF_BYCOMMAND |
	                (wp.showCmd == SW_SHOWNORMAL ? MF_ENABLED : MF_GRAYED) );

	// size is enabled only if normal
	EnableMenuItem( systemMenu, SC_SIZE, MF_BYCOMMAND |
	                (wp.showCmd == SW_SHOWNORMAL ? MF_ENABLED : MF_GRAYED) );

	// minimize is enabled only if not minimized
	EnableMenuItem( systemMenu, SC_MOVE, MF_BYCOMMAND |
	                (wp.showCmd != SW_SHOWMINIMIZED ? MF_ENABLED : MF_GRAYED) );

	// maximize is enabled only if not maximized
	EnableMenuItem( systemMenu, SC_MOVE, MF_BYCOMMAND |
	                (wp.showCmd != SW_SHOWMAXIMIZED ? MF_ENABLED : MF_GRAYED) );


	// let application modify menu
	SendMessage( hTask, WM_INITMENUPOPUP, (WPARAM) systemMenu, MAKELPARAM( 0, TRUE ) );
	SendMessage( hTask, WM_INITMENU, (WPARAM) systemMenu, 0 );

	// display the menu
	POINT pt;
	GetCursorPos( &pt );

	int command = TrackPopupMenu( systemMenu,
	                              TPM_RETURNCMD | TPM_RIGHTBUTTON,
	                              pt.x, pt.y, 0,
	                              container.hWnd,
	                              NULL );

	if ( command )
	{
		if ( command != SC_MINIMIZE && command != SC_CLOSE )
			SetForegroundWindow( hTask );

		PostMessage( hTask, WM_SYSCOMMAND, (WPARAM) command, MAKELPARAM( pt.x, pt.y ) );
	}
	else
	{
		container.Invalidate( x, y, width + 1, height + 1 );
	}
}

void TaskbarButton::MenuDown(){
	container.SetCapture( this );

	//mousePressed = TRUE;

	this->SetFlashing(FALSE);


	container.Invalidate( x, y, width + 1, height + 1 );
}

void TaskbarButton::OnMButtonDown( int, int )
{
	switch (taskbar->TaskBarTileMButton){
		case MOUSE_SELECT:
			SelectDown();
		    break;
		case MOUSE_MENU:
			MenuDown();
		    break;
		case MOUSE_ONTOP:
			OnTopToggle();
		    break;
		case MOUSE_CLOSE:
			Close();
		    break;
		case MOUSE_CLOSE2:
			Close2();
		    break;
		case MOUSE_NONE:
		    break;
        default:
		    break;
	}
}

void TaskbarButton::OnMButtonUp( int, int )
{
    
	switch (taskbar->TaskBarTileMButton){
		case MOUSE_SELECT:
			SelectUp();
		    break;
		case MOUSE_MENU:
			MenuUp();
		    break;
		case MOUSE_ONTOP:
			OnTopToggle();
		    break;
		case MOUSE_CLOSE:
			Close();
			break;
		case MOUSE_CLOSE2:
			Close2();
		    break;
		case MOUSE_NONE:
		    break;
        default:
		    break;
	}
}

void TaskbarButton::OnLButtonUp( int, int )
{
	switch (taskbar->TaskBarTileLButton){
		case MOUSE_SELECT:
			SelectUp();
		    break;
		case MOUSE_MENU:
			MenuUp();
		    break;
		case MOUSE_ONTOP:
			OnTopToggle();
			break;
		case MOUSE_CLOSE:
			Close();
		    break;
		case MOUSE_CLOSE2:
			Close2();
		    break;
		case MOUSE_NONE:
		    break;
        default:
		    break;
	}
}

void TaskbarButton::OnMouseMove( int mX, int mY )
{
	if ( mouseOver != HasPoint( mX, mY ) )
	{
		mouseOver = !mouseOver;
		container.Invalidate( x, y, width + 1, height + 1 );	
	}
		if(taskbar->switchOnHover && getCurrentState(false) != STATE_SELECTED) //20021108
		//if(getCurrentState(false) != STATE_SELECTED)
		{
			//20021110
			SendMessage( GetLitestepWnd(), LM_BRINGTOFRONT, 0, (LPARAM) hTask );
			SwitchToThisWindow( hTask, SW_RESTORE );
		}
		//buggy
		/*else if(taskbar->minimizeOnHover && getCurrentState(false) == STATE_SELECTED)
		{
			ShowWindowAsync( hTask, SW_MINIMIZE );
		}*/
}

int TaskbarButton::getCurrentState(bool returnFlashing = true){
	BOOL drawPressed = windowIsActive(this->hTask);

	if(returnFlashing && GetFlashing() && container.IsTaskFlash() && !IsOnTaskbarList(this->hTask, NoFlashList))
        return STATE_FLASHING;
	else if(IsIconic(this->hTask))
        return STATE_MINIMIZED;
	else if ( drawPressed )
        return STATE_SELECTED;
	else
		return STATE_NORMAL;
}

void TaskbarButton::OnPaint( HDC hDC )
{
	RECT r;

	r.left = x;
	r.top = y;
	r.right = x + width;
	r.bottom = y + height;

    TaskbarSkin *tempskin;

	int skinShift = 0;
	int textShift = 0;

    bool drawPressed = windowIsActive(this->hTask);

	if( drawPressed ){
		taskbar->lastActive = this->hTask;
	}

	if(container.IsTaskFlash() && drawPressed){
		this->SetFlashing(FALSE);
		flashing = false;
	}

	currentState = getCurrentState();

	if(currentState == STATE_FLASHING && !taskbar->useRealFlashing){
		tempskin = GetFlashSkin();
	}
    else if(currentState == STATE_FLASHING && taskbar->useRealFlashing && flashingFlashSkin){//20020611
		tempskin = GetFlashSkin();
	}
	else if(currentState == STATE_MINIMIZED)
		tempskin = GetMinimizedSkin();
	else if ( currentState == STATE_SELECTED )
	{
		//flash active
		tempskin = GetActiveSkin();

		skinShift = container.noSkinShift ? 0 : 1;
		textShift = container.noTextShift ? 0 : 1;
	}
	else{
		/*if(!flashingFlashSkin)
			flashingFlashSkin = true;//20020611*/
		tempskin = GetSkin();
	}


	if ( !tempskin )
	{
		if ( !container.msTaskbar )
		{
			// simple 3D frame
			HBRUSH hbrBack;
			HPEN hpnFore;
			HPEN hpnTemp;
			COLORREF backcolor;
			if(currentState == STATE_FLASHING)
				backcolor = container.flashColor;
			else if(currentState == STATE_MINIMIZED)
				backcolor = container.minColor;
			else if (currentState == STATE_SELECTED)
			{
				backcolor = container.activeColor;
			}
			else{
				backcolor = container.backColor;
			}

			hbrBack = CreateSolidBrush(backcolor);
			FillRect( hDC, &r, hbrBack );
			DeleteObject( hbrBack );

			hpnFore = CreatePen( PS_SOLID, 1, drawPressed ? container.foreColor1 : container.foreColor2 );
			hpnTemp = (HPEN) SelectObject( hDC, hpnFore );

			MoveToEx( hDC, r.left, r.bottom - 1, NULL );
			LineTo( hDC, r.right - 1, r.bottom - 1 );
			LineTo( hDC, r.right - 1, r.top );

			SelectObject( hDC, hpnTemp );
			DeleteObject( hpnFore );

			hpnFore = CreatePen( PS_SOLID, 1, drawPressed ? container.foreColor2 : container.foreColor1 );
			hpnTemp = (HPEN) SelectObject( hDC, hpnFore );

			LineTo( hDC, r.left, r.top );
			LineTo( hDC, r.left, r.bottom - 1 );

			SelectObject( hDC, hpnTemp );
			DeleteObject( hpnFore );
		}
		else
		{
			// MS taskbar style
			DrawEdge( hDC, &r, drawPressed ? EDGE_SUNKEN : EDGE_RAISED, BF_RECT | BF_MIDDLE | BF_SOFT );
		}
	}
	else
	{
		// simple 3D frame
		HBRUSH hbrBack;

		hbrBack = CreateSolidBrush( container.backColor );
		FillRect( hDC, &r, hbrBack );
		DeleteObject( hbrBack );

		// skin
		tempskin->Apply( hDC, x + skinShift, y + skinShift, width, height );
	}

	r.left += taskbar->buttonBorderLeft;
	r.top += taskbar->buttonBorderTop;
	r.right -= taskbar->buttonBorderRight;
	r.bottom -= taskbar->buttonBorderBottom;

	OffsetRect( &r, textShift, textShift );

	if (taskbar->showIcon && icon)
	{
		int iconWidth = taskbar->iconSize, iconHeight =  taskbar->iconSize;
		int tmpIconX, tmpIconY;

		if(taskbar->iconX<0)
			tmpIconX = this->width / 2 + r.left - taskbar->iconSize / 2 - taskbar->buttonBorderLeft;
		else{
			tmpIconX = r.left + taskbar->iconX;
		}

		if(taskbar->iconY<0)
			tmpIconY = this->height / 2 + r.top - taskbar->iconSize / 2 - taskbar->buttonBorderTop;
		else{
			tmpIconY = r.top + taskbar->iconY;
		}

		if (((r.right - r.left) < taskbar->iconSize) || ((r.bottom - r.top) < taskbar->iconSize))
			iconHeight = iconWidth = min(r.right - r.left, r.bottom - r.top);

		DrawIconEx( hDC, tmpIconX, tmpIconY , icon, iconWidth, iconHeight, 0, NULL, DI_NORMAL );
	}

	if(taskbar->showText){
		r.left += taskbar->buttonTextBorderLeft;
		r.top += taskbar->buttonTextBorderTop;
		r.right -= taskbar->buttonTextBorderRight;
		r.bottom -= taskbar->buttonTextBorderBottom;

		COLORREF tempColor;
		COLORREF tempShadowColor;
		HFONT tempFont;

		SetBkMode( hDC, TRANSPARENT );

		UINT position = DT_SINGLELINE | DT_END_ELLIPSIS;
		if(taskbar->textX == -1)
			position = position | DT_LEFT;
		else if(taskbar->textX == -2)
			position = position | DT_CENTER;
		else if(taskbar->textX == -3)
			position = position | DT_RIGHT;
	
		if(taskbar->textY == -1)
			position = position | DT_TOP;
		else if(taskbar->textY == -2)
			position = position | DT_VCENTER;
		else if(taskbar->textY == -3)
			position = position | DT_BOTTOM;

		bool currentUseShadow;

		if(currentState == STATE_FLASHING){
			currentUseShadow = taskbar->taskbarTextShadow && !taskbar->textFlashingNoShadow;
			tempShadowColor = SetTextColor( hDC, container.textFlashingShadowColor);
		}
		else if(currentState == STATE_MINIMIZED){
		    currentUseShadow = taskbar->taskbarTextShadow && !taskbar->textMinimizedNoShadow;
			tempShadowColor = SetTextColor( hDC, container.textMinimizedShadowColor);
		}
		else if (currentState == STATE_SELECTED){
		    currentUseShadow = taskbar->taskbarTextShadow && !taskbar->textSelectedNoShadow;
			tempShadowColor = SetTextColor( hDC, container.textSelectedShadowColor);
		}
		else{
		    currentUseShadow = taskbar->taskbarTextShadow && !taskbar->textNormalNoShadow;
			tempShadowColor = SetTextColor( hDC, container.textNormalShadowColor);
		}

		//if(taskbar->taskbarTextShadow){
		if(currentUseShadow){
			RECT r2;

			r2.bottom = r.bottom + taskbar->textShadowOffsetY;
			r2.left = r.left + taskbar->textShadowOffsetX;
			r2.right = r.right + taskbar->textShadowOffsetX;
            r2.top = r.top + taskbar->textShadowOffsetY;

//			tempShadowColor = SetTextColor( hDC, container.textShadowColor);

			if (hTask)
			{
				if (caption)
				{
  					//tempShadowColor = SetTextColor( hDC, container.textShadowColor );
					tempFont = (HFONT) SelectObject( hDC, container.activeFont );
					DrawText( hDC, caption, strlen( caption ), &r2, position );
					SetTextColor( hDC, tempShadowColor );
					SelectObject( hDC, tempFont );

				}
			}
			else
			{
				if (otherCaption)
				{
  					//tempShadowColor = SetTextColor( hDC, container.textShadowColor );
					DrawText( hDC, caption, strlen( caption ), &r2, position );
					SetTextColor( hDC, tempShadowColor );
					SelectObject( hDC, tempFont );
				}
			}

			  SetTextColor( hDC, tempColor );
			SelectObject( hDC, tempFont );
		}


		if(currentState == STATE_FLASHING)
		{
			tempColor = SetTextColor( hDC, container.textFlashColor );
			tempFont = (HFONT) SelectObject( hDC, container.flashFont );
			flashing = true;
		}
		else if(currentState == STATE_MINIMIZED)
		{
			tempColor = SetTextColor( hDC, container.textMinimizedColor );
			tempFont = (HFONT) SelectObject( hDC, container.minimizedFont );
		}
		else if (currentState == STATE_SELECTED)
		{
	        tempColor = SetTextColor( hDC, container.textActiveColor );
			tempFont = (HFONT) SelectObject( hDC, container.activeFont );
		}
		else{
			tempColor = SetTextColor( hDC, container.textColor );
			tempFont = (HFONT) SelectObject( hDC, container.font );
		}

		/*SetBkMode( hDC, TRANSPARENT );

		UINT position = DT_SINGLELINE | DT_END_ELLIPSIS;
		if(taskbar->textX == -1)
			position = position | DT_LEFT;
		else if(taskbar->textX == -2)
			position = position | DT_CENTER;
		else if(taskbar->textX == -3)
			position = position | DT_RIGHT;
	
		if(taskbar->textY == -1)
			position = position | DT_TOP;
		else if(taskbar->textY == -2)
			position = position | DT_VCENTER;
		else if(taskbar->textY == -3)
			position = position | DT_BOTTOM;*/

		if (hTask)
		{
			if (caption)
			{
				/*if(taskbar->shadow){
  		          tempShadowColor = SetTextColor( hDC, container.textShadowColor );
				  tempFont = (HFONT) SelectObject( hDC, container.activeFont );
				  DrawText( hDC, caption, strlen( caption ), &r2, position );
				  SetTextColor( hDC, tempShadowColor );
				  SelectObject( hDC, tempFont );
				}*/

				DrawText( hDC, caption, strlen( caption ), &r, position );
			}
		}
		else
		{
			if (otherCaption)
			{
				/*if(taskbar->shadow){
  		          tempShadowColor = SetTextColor( hDC, container.textShadowColor );
				  DrawText( hDC, caption, strlen( caption ), &r2, position );
				  SetTextColor( hDC, tempShadowColor );
				  SelectObject( hDC, tempFont );
				}*/
				DrawText( hDC, otherCaption, strlen( otherCaption ), &r, position );
			}
		}

		SetTextColor( hDC, tempColor );
		SelectObject( hDC, tempFont );
	}
}

void TaskbarButton::OnRButtonDown( int, int )
{
	switch (taskbar->TaskBarTileRButton){
		case MOUSE_SELECT:
			SelectDown();
		    break;
		case MOUSE_MENU:
			MenuDown();
		    break;
		case MOUSE_ONTOP:
			OnTopToggle();
			break;
		case MOUSE_CLOSE:
			Close();
		    break;
		case MOUSE_CLOSE2:
			Close2();
		    break;
		case MOUSE_NONE:
		    break;
        default:
		    break;
	}
}

void TaskbarButton::OnRButtonUp( int, int )
{
	switch (taskbar->TaskBarTileRButton){
		case MOUSE_SELECT:
			SelectUp();
		    break;
		case MOUSE_MENU:
			MenuUp();
		    break;
		case MOUSE_ONTOP:
			OnTopToggle();
			break;
		case MOUSE_CLOSE:
			Close();
		    break;
		case MOUSE_CLOSE2:
			Close2();
		    break;
		case MOUSE_NONE:
		    break;
        default:
		    break;
	}
}

bool TaskbarButton::windowIsActive(HWND htask){
	HWND tmpActive;
	tmpActive = GetWindow(GetForegroundWindow(), GW_OWNER);
	if(tmpActive)
      return (tmpActive == htask);
	else
      return (GetForegroundWindow() == htask);
}

void TaskbarButton::OnUpdate(BOOL needRedraw)
{
	//it's not a force update, so let's check if we need to redraw
	if (!needRedraw)
	{
		// has activiation state changed?

/*		bool tempActive = windowIsActive(this->hTask);

		if ( tempActive != windowIsActive(this->hTask))
		{
			needRedraw = TRUE;
		}*/

		// has caption changed?
		LPSTR temp1, temp2;

		if (hTask)
		{
			temp1 = caption;
			temp2 = GetWindowCaption();
		}
		else
		{
			temp1 = otherCaption;
			temp2 = NULL;
		}

		if ( temp2 == NULL || strcmp( temp1, temp2 ) != 0 )
		{
			if (temp2)
			{
				delete [] caption;
				caption = temp2;
				needRedraw = TRUE;
				temp1 = caption;
			}

			// update tooltip
			TOOLINFO ti;
			memset( &ti, 0, sizeof(TOOLINFO) );

			ti.cbSize = sizeof(TOOLINFO);
			ti.hwnd = container.hWnd;
			ti.uId = (UINT) this;
			ti.lpszText = temp1;

			//20021108
			if(container.useToolTips)
			{
				SendMessage( container.hToolTips, TTM_UPDATETIPTEXT, 0, (LPARAM) &ti );
				SetWindowPos( container.hToolTips, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);
			}
		}
		else
		{
			if (temp2)
			{
				delete [] temp2;
			}
		}


		// has icon changed?
		HICON tempIcon = GetIcon();

		if ( icon != tempIcon )
		{
			icon = tempIcon;
			needRedraw = TRUE;
		}
	}

	if ( needRedraw )
	  container.Invalidate( x, y, width + 1, height + 1 );
}

LPSTR TaskbarButton::GetWindowCaption()
{
	LPSTR windowCaption = NULL;
	int length;

	if (hTask)
	{
		length = GetWindowTextLength( hTask );

		windowCaption = new char[length + 1];
		GetWindowText( hTask, windowCaption, length + 1 );
	}

	return windowCaption;
}

HICON TaskbarButton::GetIcon()
{
	return GetIconFromWindow( hTask, (taskbar->iconSize>=32) ); //get a big icon sometimes
}

void TaskbarButton::SetSkin(TaskbarSkin *s)
{
	skin = s;
}

void TaskbarButton::SetActiveSkin(TaskbarSkin *s)
{
	activeskin = s;
}

void TaskbarButton::SetFlashSkin(TaskbarSkin *s)
{
	flashskin = s;
}

void TaskbarButton::SetMinimizedSkin(TaskbarSkin *s)
{
	minimizedskin = s;
}

void TaskbarButton::SetCaption(LPCSTR cap)
{
	if (!cap)
	{
		return ;
	}

	if (otherCaption)
	{
		delete [] otherCaption;
	}

	otherCaption = new char[strlen(cap) + 1];

	strcpy(otherCaption, cap);

	// add tooltip
	TOOLINFO ti;
	memset( &ti, 0, sizeof(TOOLINFO) );

	ti.cbSize = sizeof(TOOLINFO);
	ti.hwnd = container.hWnd;
	ti.uId = (UINT) this;
	ti.lpszText = otherCaption;

	//20021108
	if(container.useToolTips)
	{
		SendMessage( container.hToolTips, TTM_ADDTOOL, 0, (LPARAM) &ti );
		SetWindowPos( container.hToolTips, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);
	}
}
